@@ -4,6 +4,7 @@ BEGIN { die "Your perl version is old, see README for instructions" if $] < 5.00
 use strict;
 use vars qw($VERSION %DEFAULTS @ISA @EXPORT @EXPORT_OK);
 use Carp;
+use Data::Dumper;
 
 require Exporter;
 require AutoLoader;
@@ -15,7 +16,7 @@ require AutoLoader;
 @EXPORT = qw(
 	
 );
-$VERSION = '1.54';
+$VERSION = '1.57';
 %DEFAULTS = (
   "CSV_DELIMITER"=>',', # controls how to read/write CSV file
   "CSV_QUALIFIER"=>'"',
@@ -24,6 +25,11 @@ $VERSION = '1.54';
   # (\r\n as linebreak), 2 for MAC  (\r as linebreak)
   # this controls how to read and write CSV/TSV file
 );
+%Data::Table::TSV_ESC = ( '0'=>"\0", 'n'=>"\n", 't'=>"\t", 'r'=>"\r", 'b'=>"\b",
+              "'"=>"'", '"'=>"\"", '\\'=>"\\" );
+%Data::Table::TSV_ENC = ( "\0"=>'0', "\n"=>'n', "\t"=>'t', "\r"=>'r', "\b"=>'b',
+              "'"=>"'", "\""=>'"', "\\"=>'\\' );
+ 
 
 sub new {
   my ($pkg, $data, $header, $type, $enforceCheck) = @_;
@@ -105,11 +111,11 @@ sub csvEscape {
 
 sub tsvEscape {
   my $s = shift;
-  my %ESC = ( "\0"=>'0', "\n"=>'n', "\t"=>'t', "\r"=>'r', "\b"=>'b',
-              "'"=>"'", "\""=>'"', "\\"=>'\\' );
+  #my %ESC = ( "\0"=>'0', "\n"=>'n', "\t"=>'t', "\r"=>'r', "\b"=>'b',
+  #            "'"=>"'", "\""=>'"', "\\"=>'\\' );
   ## what about \f? MySQL treats \f as f.
   return "\\N" unless defined($s);
-  $s =~ s/([\0\\\b\r\n\t"'])/\\$ESC{$1}/g;
+  $s =~ s/([\0\\\b\r\n\t"'])/\\$Data::Table::TSV_ENC{$1}/g;
   return $s;
 }
 
@@ -158,18 +164,31 @@ sub tsv {
   my ($self, $header, $arg_ref)=@_;
   my ($status, @t);
   my $s = '';
-  my ($OS, $fileName_or_handler) = ($Data::Table::DEFAULTS{OS}, undef);
-  $OS = $arg_ref->{'OS'} if (defined($arg_ref) && defined($arg_ref->{'OS'}));
-  $fileName_or_handler = $arg_ref->{'file'} if (defined($arg_ref) && defined($arg_ref->{'file'}));
+  my ($OS, $fileName_or_handler, $transform_element) = ($Data::Table::DEFAULTS{OS}, undef, 1);
+  if (defined($arg_ref)) {
+    $OS = $arg_ref->{'OS'} if (defined($arg_ref->{'OS'}));
+    $fileName_or_handler = $arg_ref->{'file'} if (defined($arg_ref->{'file'}));
+    $transform_element = $arg_ref->{'transform_element'} if (defined($arg_ref->{'transform_element'}));
+  }
   my $endl = ($OS==2)?"\r":(($OS==1)?"\r\n":"\n");
   $header=1 unless defined($header);
-  $s=join("\t", map {tsvEscape($_)} @{$self->{header}}) . $endl if $header;
+  if ($header) {
+    if ($transform_element) {
+      $s=join("\t", map {tsvEscape($_)} @{$self->{header}}) . $endl;
+    } else {
+      $s=join("\t",@{$self->{header}}) . $endl;
+    }
+  }
 ######  $self->rotate if $self->{type};
   if ($self->{data}) {
     $self->rotate() if ($self->{type});
     my $data=$self->{data};
     for (my $i=0; $i<=$#{$data}; $i++) {
-      $s .= join("\t", map {tsvEscape($_)} @{$data->[$i]}) . $endl;
+      if ($transform_element) {
+        $s .= join("\t", map {tsvEscape($_)} @{$data->[$i]}) . $endl;
+      } else {
+        $s .= join("\t", @{$data->[$i]}) . $endl;
+      }
     }
   }
   if (defined($fileName_or_handler)) {
@@ -997,7 +1016,14 @@ sub parseCSV {
   my $delimiter2 = $delimiter; $delimiter2 = substr($delimiter, 1, 1) if length($delimiter)>1;
   my $qualifier2 = $qualifier; $qualifier2 = substr($qualifier, 1, 1) if length($qualifier)>1;
   # $s =~ s/\n$//; # chop" # assume extra characters has been cleaned before
-  return [split /$delimiter/, $s , $size] if -1==index $s, $qualifier;
+  if (-1==index $s, $qualifier) {
+    if ($size == 0) {
+      my $s2 = $s;
+      $s2 =~ s/$delimiter//g;
+      $size = length($s)-length($s2)+1;
+    }
+    return [split /$delimiter/, $s , $size];
+  }
   $s =~ s/\\/\\\\/g; # escape \ => \\
   my $n = length($s);
   my ($q, $i)=(0, 0);
@@ -1016,6 +1042,11 @@ sub parseCSV {
   $s =~ s/$qualifier\s*$delimiter/$delimiter2/g;
   $s =~ s/$delimiter\s*$qualifier/$delimiter2/g;
   $s =~ s/$qualifier$qualifier/$qualifier2/g;
+  if ($size == 0) {
+    my $s2 = $s;
+    $s2 =~ s/$delimiter//g;
+    $size = length($s)-length($s2)+1;
+  }
   my @parts=split(/$delimiter/, $s, $size);
   @parts = map {$_ =~ s/(\\c|\\\\)/$1 eq '\c'?$delimiter2:'\\'/eg; $_ } @parts;
 #  my @parts2=();
@@ -1027,6 +1058,19 @@ sub parseCSV {
   return \@parts;
 }
 
+sub transformElement {
+  my $one = shift;
+  for (my $i=0; $i < scalar @$one; $i++) {
+    next unless defined($one->[$i]);
+    if ($one->[$i] eq "\\N") {
+      $one->[$i]=undef;
+    } else {
+      $one->[$i] =~ s/\\([0ntrb'"\\])/$Data::Table::TSV_ESC{$1}/g;
+    }
+  }
+  return $one;
+}
+
 sub fromTSVi {
   my $self = shift;
   return fromTSV(@_);
@@ -1034,15 +1078,15 @@ sub fromTSVi {
 
 sub fromTSV {
   my ($name_or_handler, $includeHeader, $header, $arg_ref) = @_;
-  my ($OS, $skip_lines, $skip_pattern) = ($Data::Table::DEFAULTS{OS}, 0, undef);
+  my ($OS, $skip_lines, $skip_pattern, $transform_element) = ($Data::Table::DEFAULTS{OS}, 0, undef, 1);
   $OS = $arg_ref->{'OS'} if (defined($arg_ref) && defined($arg_ref->{'OS'}));
   # OS: 0 for UNIX (\n as linebreak), 1 for Windows (\r\n as linebreak)
   ###   2 for MAC  (\r as linebreak)
   $skip_lines = $arg_ref->{'skip_lines'} if (defined($arg_ref) && defined($arg_ref->{'skip_lines'}) && $arg_ref->{'skip_lines'}>0);
   $skip_pattern = $arg_ref->{'skip_pattern'} if defined($arg_ref->{'skip_pattern'});
- 
-  my %ESC = ( '0'=>"\0", 'n'=>"\n", 't'=>"\t", 'r'=>"\r", 'b'=>"\b",
-              "'"=>"'", '"'=>"\"", '\\'=>"\\" );
+  $transform_element = $arg_ref->{'transform_element'} if (defined($arg_ref->{'transform_element'}));
+  #my %ESC = ( '0'=>"\0", 'n'=>"\n", 't'=>"\t", 'r'=>"\r", 'b'=>"\b",
+  #            "'"=>"'", '"'=>"\"", '\\'=>"\\" );
   ## what about \f? MySQL treats \f as f.
 
   $includeHeader = 1 unless defined($includeHeader);
@@ -1094,27 +1138,34 @@ sub fromTSV {
   my $size = scalar @$one;
   unless ($givenHeader) {
     if ($includeHeader) {
-      @header = map { $_ =~ s/\\([0ntrb'"\\])/$ESC{$1}/g; $_ } @$one;
+      if ($transform_element) {
+        @header = map { $_ =~ s/\\([0ntrb'"\\])/$Data::Table::TSV_ESC{$1}/g; $_ } @$one;
+      } else {
+        @header = @$one;
+      }
     } else {
       @header = map {"col$_"} (1..$size); # name each column as col1, col2, .. etc
     }
   }
-  push @data, $one unless ($includeHeader);
-
+  unless ($includeHeader) {
+    transformElement($one) if $transform_element;
+    push @data, $one;
+  }
   while($s = <$SRC>) {
     #chop;
     # $_=~ s/$newRowDelimiter$//;
     next if (defined($skip_pattern) && $s =~ /$skip_pattern/);
     if (substr($s, -$n_endl, $n_endl) eq $newRowDelimiter) { for (1..$n_endl) { chop $s }}
     my @one = split(/\t/, $s, $size);
-    for (my $i=0; $i < $size; $i++) {
-      next unless defined($one[$i]);
-      if ($one[$i] eq "\\N") {
-        $one[$i]=undef;
-      } else {
-        $one[$i] =~ s/\\([0ntrb'"\\])/$ESC{$1}/g;
-      }
-    }
+    transformElement(\@one) if $transform_element;
+    #for (my $i=0; $i < $size; $i++) {
+    #  next unless defined($one[$i]);
+    #  if ($one[$i] eq "\\N") {
+    #    $one[$i]=undef;
+    #  } else {
+    #    $one[$i] =~ s/\\([0ntrb'"\\])/$Data::Table::TSV_ESC{$1}/g;
+    #  }
+    #}
     confess "Inconsistent column number at data entry: ".($#data+1) unless ($size==scalar @one);
     push @data, \@one;
   }
@@ -1186,7 +1237,16 @@ sub join {
   my @subRow;
   for ($i = 0; $i < $self->nofRow; $i++) {
     @subRow = @{$data1->[$i]}[@$cols1];
-    $key = join("\t", map {tsvEscape($_)} @subRow);
+    # set $key to '\\N' as long as one of the key fields is undef
+    my @S = map {tsvEscape($_)} @subRow;
+    for ($j=0; $j<@S; $j++) {
+      last if ($S[$j] eq '\\N');
+    }
+    if ($j>= @S) {
+      $key = join("\t", @S);
+    } else {
+      $key = '\\N';
+    }
     unless (defined($H{$key})) {
       $H{$key} = [[$i], []];
     } else {
@@ -1195,7 +1255,20 @@ sub join {
   }
   for ($i = 0; $i < $tbl->nofRow; $i++) {
     @subRow = @{$data2->[$i]}[@$cols2];
-    $key = join("\t", map {tsvEscape($_)} @subRow);
+    # set $key to '\\N\\N' as long as one of the key fields is undef
+    # we intentionally make the second table undef keys to be '\\N\\N',
+    # so that they are different from the first table undef keys
+    # avoid NULL == NULL in the join
+    my @S = map {tsvEscape($_)} @subRow;
+
+    for ($j=0; $j<@S; $j++) {
+      last if ($S[$j] eq '\\N');
+    }
+    if ($j>= @S) {
+      $key = join("\t", @S);
+    } else {
+      $key = '\\N\\N';
+    }
     unless (defined($H{$key})) {
       $H{$key} = [[], [$i]];
     } else {
@@ -1350,7 +1423,8 @@ sub pivot {
   my $y = $self->checkOldCol($colToSplit);
   my $y_name = defined($y)?$self->{header}->[$y]:undef;
   confess "Unknown column ". $colToSplit if (!defined($y) && defined($colToSplit));
-  my $z = $self->checkOldCol($colToFill);
+  my $z = undef;
+  $z = $self->checkOldCol($colToFill) if defined($colToFill);
   my $z_name = defined($z)?$self->{header}->[$z]:undef;
   confess "Unknown column ". $colToFill if (!defined($z) && defined($colToFill));
   confess "Cannot take colToFill, if colToSplit is 'undef'" if (defined($z) && !defined($y));
@@ -1539,22 +1613,35 @@ sub fromFileGuessDelimiter {
 sub fromFile {
   my ($name, $arg_ref) = @_;
   my $linesChecked = 2;
-  $linesChecked= $arg_ref->{'OS'} if (defined($arg_ref) && defined($arg_ref->{'OS'}));
+  my $os = undef;
+  my $hasHeader = undef;
+  my $delimiter = undef;
   if (defined($arg_ref)) {
     $linesChecked = $arg_ref->{'linesChecked'} if defined($arg_ref->{'linesChecked'});
+    $os = $arg_ref->{'OS'};
+    $hasHeader = $arg_ref->{'has_header'};
+    $delimiter = $arg_ref->{'delimiter'};
+  }
+  unless (defined($os)) {
+    $os = fromFileGuessOS($name);
+    $arg_ref->{'OS'}=$os;
   }
-  my $os = fromFileGuessOS($name);
   my @S = fromFileGetTopLines($name, $os, $linesChecked);
   return undef unless scalar @S;
-  my $delimiter = fromFileGuessDelimiter(\@S);
-  my $hasHeader = fromFileIsHeader($S[0], $delimiter);
+  unless (defined($delimiter)) {
+    $delimiter = fromFileGuessDelimiter(\@S);
+    $arg_ref->{'delimiter'} = $delimiter;
+  }
+  unless (defined($hasHeader)) {
+    $hasHeader = fromFileIsHeader($S[0], $delimiter);
+  }
   my $t = undef;
   #print ">>>". join("\n", @S)."\n";
   #print "OS=$os, hasHeader=$hasHeader, delimiter=$delimiter\n";
   if ($delimiter eq "\t") {
-    $t=fromTSV($name, $hasHeader, undef, {OS=>$os});
+    $t=fromTSV($name, $hasHeader, undef, $arg_ref);
   } else {
-    $t=fromCSV($name, $hasHeader, undef, {OS=>$os, delimiter=>$delimiter});
+    $t=fromCSV($name, $hasHeader, undef, $arg_ref);
   }
   return $t;
 }
@@ -1867,7 +1954,7 @@ The following example reads bbb.csv file (included in the package) by skipping t
 
 Same as Data::Table::fromCSV. However, this is an instant method (that's what 'i' stands for), which can be inherited.
 
-=item table Data::Table::fromTSV ($name, $includeHeader = 1, $header = ["col1", ... ], {OS=>$Data::Table::DEFAULTS{'OS'}, skip_lines=>0, skip_pattern=>undef})
+=item table Data::Table::fromTSV ($name, $includeHeader = 1, $header = ["col1", ... ], {OS=>$Data::Table::DEFAULTS{'OS'}, skip_lines=>0, skip_pattern=>undef, transform_element=>1})
 
 create a table from a TSV file.
 return a table object.
@@ -1881,6 +1968,8 @@ optional name argument skip_lines let you specify how many lines in the csv file
 
 optional name argument skip_pattern let you specify a regular expression. Lines that match the regular expression will be skipped.
 
+optional name argument transform_element let you switch on/off \t to tab, \N to undef (etc.) transformation. See TSV FORMAT for details. However, elements are always transformed when export table to tsv format, because not escaping an element containing a tab will be disasterous.
+
 See similar examples under Data::Table::fromCSV;
 
 Note: read "TSV FORMAT" section for details.
@@ -1889,12 +1978,13 @@ Note: read "TSV FORMAT" section for details.
 
 Same as Data::Table::fromTSV. However, this is an instant method (that's what 'i' stands for), which can be inherited.
 
-=item table Data::Table::fromFile ($file_name, {linesChecked=>2})
+=item table Data::Table::fromFile ($file_name, $arg_ref = {linesChecked=>2})
 
 create a table from a text file.
 return a table object.
 $file_name: the file name (cannot take a file handler).
 linesChecked: the first number of lines used for guessing the input format. The delimiter will have to produce the same number of columns for these lines. By default only check the first 2 lines, 0 means all lines in the file.
+$arg_ref can take additional parameters, such as OS, has_header, delimiter, transform_element, etc.
 
 fromFile is added after version 1.51. It relies on the following new methods to automatically figure out the correct file format in order to call fromCSV or fromTSV internally:
 
@@ -1907,7 +1997,7 @@ fromFile is added after version 1.51. It relies on the following new methods to
   fromFileIsHeader($line_concent, $delimiter) # $delimiter defaults to $Data::Table::DEFAULTS{'CSV_DELIMITER'}
     returns 1 or 0.
 
-It first ask fromFileGuessOS to figure out which OS (UNIX, PC or MAC) generated the input file. The fetch the first linesChecked lines using fromFileGetTopLines. It then guesses the best delimiter using fromFileGuessDelimiter, then it checks if the first line looks like a column header row using fromFileIsHeader. Since fromFileGuessOS and fromFileGetTopLines needs to open/close the input file, these methods can only take file name, not file handler.
+It first ask fromFileGuessOS to figure out which OS (UNIX, PC or MAC) generated the input file. The fetch the first linesChecked lines using fromFileGetTopLines. It then guesses the best delimiter using fromFileGuessDelimiter, then it checks if the first line looks like a column header row using fromFileIsHeader. Since fromFileGuessOS and fromFileGetTopLines needs to open/close the input file, these methods can only take file name, not file handler. If user specify formatting parameters in $arg_ref, the routine will skip the corresponding guess work. At the end, fromFile simply calls either fromCSV or fromTSV with $arg_ref forwarded. So if you call fromFile({transform_element=>0}) on a TSV file, transform_elment will be passed onto fromTSV calls internally.
 
 fromFileGuessOS finds the linebreak that gives shortest first line (in the priority of UNIX, PC, MAC upon tie).
 fromFileGuessDelimiter works based on the assumption that the correct delimiter will produce equal number of columns for the given rows. If multiple matches, it chooses the delimiter that gives maximum number of columns. If none matches, it returns the default delimiter.
@@ -2241,7 +2331,7 @@ This creates a 2x3 table, where Departments are use as row keys, Sex (female and
 =item int table::rowMerge ($tbl)
 
 Append all the rows in the table object $tbl to the original rows.
-The merging table $tbl must have the same number of columns as the original.
+The merging table $tbl must have the same number of columns as the original, as well as the columns are in exactly the same order.
 It returns 1 upon success, undef otherwise.
 The table object $tbl should not be used afterwards, since it becomes part of
 the new table.
@@ -2283,13 +2373,17 @@ would like to understand the code and may derive a new class from Data::Table.
 convert the internal structure of a table between row-based and column-based.
 return 1 upon success, undef otherwise.
 
-=item string csvEscape($rowRef)
+=item string csvEscape($string, {delimiter=>, qualifier})
 
-Encode an array of scalars into a CSV-formatted string.
+Encode a scalar into a CSV-formatted field.
 
 optional named arguments: delimiter and qualifier, in case user wants to use characters other than the defaults. 
 The default delimiter and qualifier is taken from $Data::Table::DEFAULTS{'CSV_DELIMITER'} (defaults to ',') and $Data::Table::DEFAULTS{'CSV_QUALIFIER'} (defaults to '"'), respectively.
 
+Please note that this function only escape one element in a table. To escape the whole table row, you need to
+  join($delimiter, map {csvEscape($_)} @row . $endl;
+$endl refers to End-of-Line, which you may or may not want to add, and it is OS-dependent. Therefore, csvEscape method is kept to the simplest form as an element transformer.
+
 =item refto_array parseCSV($string)
 
 Break a CSV encoded string to an array of scalars (check it out, we did it the cool way).
@@ -2300,7 +2394,7 @@ respectively. The default delimiter and qualifier is taken from $Data::Table::DE
 
 =item string tsvEscape($rowRef)
 
-Encode an array of scalars into a TSV-formatted string.
+Encode a scalar into a TSV-formatted string.
 
 =back
 
@@ -2314,6 +2408,9 @@ We define TSV based on MySQL convention.
   (Warning: MySQL treats '\f' as 'f', and it's not escaped here)
   Undefined values are represented as '\N'.
 
+However, you can switch off this transformation by setting {transform_element => 0} in the fromTSV or tsv method.
+Before if a cell reads 'A line break is \n', it is read in as 'A link break is [return]' in memory. When use tsv method to export, it is transformed back to 'A line break is \n'. However, if it is exported as a csv, the [return] will break the format. Now if transform_element is set to 0, the cell is stored as 'A line break is \n' in memory, so that csv export will be correct. However, do remember to set {transform_element => 0} in tsv export method, otherwise, the cell will become 'A line break is \\n'. Be aware that trasform_element controls column headers as well.
+
 =head1 INTERFACE TO OTHER SOFTWARES
 
 Spreadsheet is a very generic type, therefore Data::Table class provides an easy
@@ -2341,11 +2438,6 @@ Here is a summary (partially repeat) of some classic usages of Data::Table.
 
   Same for TSV
 
-=head2 Interface to Excel
-
-  Convert between an Excel file and tables
-  see Data::Table::Excel
-
 =head2 Interface to Graphics Package
 
   use GD::Graph::points;
@@ -2373,7 +2465,7 @@ Perl.
 
 =head1 SEE ALSO
 
-  Data::Table::Excel, DBI, GD::Graph.
+  DBI, GD::Graph.
 
 =cut
 
